import datetime

from django.conf import settings
from django.test import TestCase
from model_bakery import baker

from .main.models import SelfRegisteredSubject


class TestSelfRegistrationApiView(TestCase):
    def setUp(self):
        self.client.defaults['HTTP_AUTHORIZATION'] = 'token ' + settings.API_TOKEN

    def test_get(self):
        url = '/api/subjects/'
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_confirmed_subject_in_response(self):
        subject = baker.make(
            SelfRegisteredSubject,
            date_of_birth=datetime.date(2000, 1, 1),
            confirmed=True
        )
        url = '/api/subjects/'
        response = self.client.get(url)
        data = {'subjects': [{
            'first_name': subject.first_name,
            'last_name': subject.last_name,
            'gender': subject.gender,
            'email': subject.email,
            'phone_number': str(subject.phone_number),
            'date_of_birth': str(subject.date_of_birth),
            'delete_url': 'http://testserver/' + 'api/subjects/' + '?id={}'.format(subject.pk)
            }]}
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(
            str(response.content, encoding='utf8'),
            data
        )

    def test_unconfirmed_subject_not_in_response(self):
        baker.make(
            SelfRegisteredSubject,
            date_of_birth=datetime.date(2000, 1, 1),
            confirmed=False
        )
        url = '/api/subjects/'
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'{"subjects": []}')

    def test_delete_deletes_confirmed_subjects(self):
        subject = baker.make(
            SelfRegisteredSubject,
            date_of_birth=datetime.date(2000, 1, 1),
            confirmed=True
        )
        url = '/api/subjects/?id={}'.format(subject.pk)
        response = self.client.delete(url)
        self.assertEqual(response.status_code, 204)
        self.assertFalse(SelfRegisteredSubject.objects.exists())

    def test_delete_doesnt_delete_unconfirmed_subjects(self):
        subject = baker.make(
            SelfRegisteredSubject,
            date_of_birth=datetime.date(2000, 1, 1),
            confirmed=False
        )
        url = '/api/subjects/?id={}'.format(subject.pk)
        response = self.client.delete(url)
        self.assertEqual(response.status_code, 404)
        self.assertTrue(SelfRegisteredSubject.objects.filter(pk=subject.pk).exists())
